%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% The purspose of this code is to                                     %%%
%%%     (1) Read a tif file containing multi-channel image from a       %%%
%%%         selected anatomical location                                %%%
%%%     (2) Read-out voxel intensities from the image to scatter plots  %%%
%%%         of normalized voxel intensities for pairs of target mRNAs.  %%%
%%%     (3) Select well-defined expression clusters with differing      %%%
%%%         slopes and amplitudes for a pair of target mRNAs            %%%
%%%     (4) Read-in from the selected expression clusters for a pair of %%%
%%%         target mRNAs back into the embryo, segmenting the image     %%%
%%%         into expression domains.                                    %%%
%%%                                                                     %%%
%%%                                                                     %%%
%%% This code is written by                                             %%%
%%% Vikas Trivedi, PhD                                                  %%%
%%% Department of Genetics, University of Cambridge                     %%%
%%% Downing Street,Cambridge CB2 3EH                                    %%%
%%% Phone: +44 (0) 1223 766334                                          %%%
%%% E-Mail: vikas.trivedi@gen.cam.ac.uk                                 %%%
%%%     Previous Affiliation :                                          %%%
%%%     Division of Biology & Biological Engineering,                   %%%
%%%     California Institute of Technology, Pasadena, CA 91125          %%%
%%%                                                                     %%%
%%% Please feel free to contact me if you have any questions            %%%
%%%                                                                     %%%
%%% This code is supplied with the paper :                              %%%
%%% Multidimensional quantitative analysis of mRNA expression           %%%
%%% within intact vertebrate embryos                                    %%%
%%%                                                                     %%%
%%% Vikas Trivedi, Harry M.T. Choi, Scott E. Fraser, Niles A. Pierce    %%%
%%%                                                                     %%%
%%% Copyright (c)2017, Vikas Trivedi                                    %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Revision history:
%%% 2016/06/02 VT Created the document
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%% Clearing all variables in memory
clc
clf
clear all
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ?  Script setup (Taking inputs about the image to be read)

display('Script readoutreadin.m is running')

prompt={'Multi- channel Image file (.tif) name','Number of analysis channels to be used for read-out/read-in','Index for each analysis channel','Label for each analysis channel','Maximum intensity value for each analysis channel','Minimum(background) intensity value for each analysis channel','Voxel size (number of pixels per voxel) in x, y, z '};
prompt_title = 'Script setup';
defaultans = {'zfish.tif','4','[1,2,3,4]','myod1,tpm3,her1,her7','[2062, 1943, 1713, 1538]','[31, 59, 21, 27]','[3, 3, 1]'};
num_lines = 1;
options.Interpreter = 'tex';
answer1 = inputdlg(prompt,prompt_title,num_lines,defaultans,options);

filename = answer1{1}; % This is the input image
Mchannels = str2num(answer1{2}); % Specify the number of analysis channels to be used for read-out/read-in: must be less than or equal to the number of channels in the image file
Chindices = str2num(answer1{3}); % [myoD, tpm3, her1, her7] channel indices in the image file specified above. In general this allows you to set each analysis channel to be any channel in the image file
Chlabels_temp = answer1{4}; % Labels for all the channels in the image
maxvalueChs = str2num(answer1{5});% Maximum intensity value for each analysis channel (here for [myoD, tpm3, her1, her7]): used for data normalization; for a given analysis channel, use the same value across all images in a study to enable quantitative comparisons between images (see Section S1.4.3 and Table S3)
minvalueChs = str2num(answer1{6});% Minimum (background) intensity value for each analysis channel (here for [myoD, tpm3, her1, her7]): used for data normalization; for a given analysis channel, use the same value across all images in a study to enable quantitative comparisons between images (see Section S1.4.3 and Table S3)
Voxelsize_xyz = str2num(answer1{7});
Voxelsize = 1/Voxelsize_xyz(1); % Where imresize(A, Voxelsize) returns image B that is Voxelsize times the size of A

%******************************************************** 
%******** Finding individual channel labels from the concatenated string Chlabels_temp
CommaIndices = find(Chlabels_temp==',');
indexstart = 1;
Chlabels = {};
for m  = 1:Mchannels-1
    name = Chlabels_temp(indexstart:CommaIndices(m)-1);
    Chlabels{m} = name;
    indexstart = CommaIndices(m)+1;
end

Chlabels{Mchannels} = Chlabels_temp(indexstart:length(Chlabels_temp));

clear Chlabels_temp, clear indexstart , clear CommaIndices

display('')
display('Step: Script setup .... Completed')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Specifying figure layout and other parameters for the code

figurewidthinches = 8;
figureheightinches = 8;

Displayrange = [0 1500]; % Display range for showing images
RawScatterplotrange = [0 4096];

dotsize = 0.5;
dotsymbol = '.';

NScatterPlots = nchoosek(Mchannels,2); % Number of sctter plots to be generated based on pairwise combinations of N channels
filenameprefix = filename(1:length(filename)-4);

cc_channels = jet(10); % this is assuming that someone will generate a 10 colored image
cc_channels(1,:) = [1 0 0];
cc_channels(2,:) = [0 1 0];
cc_channels(3,:) = [0 0 1];
%cc_channels(4,:) = [0.9 0.9 0];
cc_channels(4,:) = [255 215 0]/255;
cc_channels(5,:) = [1 0 1];
cc_channels(6,:) = [0 1 1];
cc_channels(7,:) = [1 1 1];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ?  Data Normalization.
A=char(filename);
info = imfinfo(A);
Ch1 = imread(A, Chindices(1), 'Info', info);%Image in channel 1 (preferably myoD)
%ChA_normalized = zeros(size(Ch1,1),size(Ch1,2),Mchannels); % This stores the normalized channel voxel values for all teh selected ROIs (boxes)

% Showing all the channels
for m = 1:Mchannels
    
    ImageA = imread(A, Chindices(m), 'Info', info);
    
    ImageAv = imresize(double(ImageA),Voxelsize); % Resizing the Channel m box depending upon the voxel size chosen
    ImageA_nv = (ImageAv - minvalueChs(m))/(maxvalueChs(m) - minvalueChs(m)); % Resizing the Channel m box depending upon the voxel size chosen
    
    figure(1) % raw image. Depicts signal + background for pixels in each analysis channel plus a merge of all analysis channels
    set(gcf,'Units','inches')
    pos = get(gcf,'pos');
    set(gcf,'pos',[pos(1) pos(2) figurewidthinches 5])
    subplot(1,Mchannels+1,m), imshow(ImageAv,Displayrange,'Border', 'tight'), hold on, title(strcat('Ch',num2str(m),':',Chlabels{m}),'color',cc_channels(m,:))
    
    figure(2) % normalized image. Depicts normalized signal estimate for voxels in each analysis channel plus a merge of all analysis channels 
    set(gcf,'Units','inches')
    pos = get(gcf,'pos');
    set(gcf,'pos',[pos(1) pos(2) figurewidthinches 5])
    subplot(1,Mchannels+1,m), imshow(ImageA_nv,[0 1],'Border', 'tight'), hold on,title(strcat('Ch',num2str(m),':',Chlabels{m}),'color',cc_channels(m,:))
    
end

%%%%%%%% 
%%% Generating merge of channels

ChA_RGB1 = zeros(size(Ch1,1),size(Ch1,2)); % This stores the Red channel in the RGB image for showing overlayed image
ChA_RGB2 = zeros(size(Ch1,1),size(Ch1,2)); % This stores the Green channel in the RGB image for showing overlayed image
ChA_RGB3 = zeros(size(Ch1,1),size(Ch1,2)); % This stores the Blue  channel in the RGB image for showing overlayed image

for m = 1:Mchannels
    ImageA = double(imread(A, Chindices(m), 'Info', info));
    
    if (m ==1)
        ChA_RGB1 = ChA_RGB1 + ImageA;
    elseif (m == 2)
        ChA_RGB2 = ChA_RGB2 + ImageA;
    elseif (m == 3)
        ChA_RGB3 = ChA_RGB3 + ImageA;
    elseif (m==4)
        ChA_RGB1 = ChA_RGB1 + ImageA;
        ChA_RGB2 = ChA_RGB2 + (215/255)*ImageA;
    end
    
end

if (numel(find(ChA_RGB1)>0)>0)
    ChA_RGB1 = ChA_RGB1/(max(ChA_RGB1(:)));
end

if (numel(find(ChA_RGB2)>0)>0)
    ChA_RGB2 = ChA_RGB2/(max(ChA_RGB2(:)));
end

if (numel(find(ChA_RGB3)>0)>0)
    ChA_RGB3 = ChA_RGB3/(max(ChA_RGB3(:)));
end

Ch_merge = cat(3, ChA_RGB1, ChA_RGB2, ChA_RGB3);

clear ChA_RGB1, clear ChA_RGB2, clear ChA_RGB3

%%%%%%%% 



figure(1)
subplot(1,Mchannels+1,Mchannels+1), imshow(Ch_merge,[],'Border', 'tight'), hold on, title ('Merge')
saveas(gcf, strcat(filenameprefix,'-01-rawimage.pdf')); %Example: Saved as: zfish-rawimage.pdf

figure(2)
subplot(1,Mchannels+1,Mchannels+1), imshow(Ch_merge,[],'Border', 'tight'), hold on, title ('Merge')
saveas(gcf, strcat(filenameprefix,'-02-image.pdf')); %Example: Saved as: zfish-image.pdf

display('')
display('Step: Data Normalization .... Completed')
display(strcat('File :',filenameprefix,'-01-rawimage.pdf is saved'))
display(strcat('File :',filenameprefix,'-02-image.pdf is saved'))



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ?  Read-out from anatomical space to expression space.


%******************************************************** 
%******** Select the number of anatomical regions of interest

prompt={'Number of anatomical regions of interest (ROIs) that you would like to select','Width (in pixel) of the ROIs','Height (in pixel) of the ROIs'};
prompt_title = 'Select anatomical regions of interest';
defaultans = {'3','60','100'};
num_lines = 1;
options.Interpreter = 'tex';
answer2 = inputdlg(prompt,prompt_title,num_lines,defaultans,options);

Nboxes = str2num(answer2{1}); % Number of ROIs (boxes) that you will draw in the image
w= str2num(answer2{2}); % Width in pixels of your ROIs
h= str2num(answer2{3});% heigh in pixels for your ROIs


cc_boxes = copper(Nboxes); %% Color code for showing scatter points from different boxes
cc_boxes(1,:) = [150 150 150]/255;

x_boxes = zeros(Nboxes,1);
y_boxes = zeros(Nboxes,1);


ChSIG = zeros(ceil(w*Voxelsize)*ceil(h*Voxelsize),Mchannels,Nboxes); % This stores the raw channel voxel values for all the selected ROIs (boxes)
ChSIGn = zeros(ceil(w*Voxelsize)*ceil(h*Voxelsize),Mchannels,Nboxes); % This stores the normalized channel voxel values for all the selected ROIs (boxes)


%******************************************************** 
%******** Select the anatomical regions of interest by drawing boxes

    figure(3)
    set(gcf,'Units','inches')
    pos = get(gcf,'pos');
    set(gcf,'pos',[pos(1) pos(2) figurewidthinches figureheightinches])
    imshow(Ch_merge,[],'Border', 'tight'), hold on
    
for i=1:Nboxes;

    figure(3)
    curr_fig = imgcf; % Place the Topleft corner of the box
    [x,y] = getpts(curr_fig); % Recording the x,y coordinates of the top left corner of the box
    rectangle('Position',[x,y,w,h], 'LineWidth', 2, 'EdgeColor',cc_boxes(i,:)) % Showing the box
    text(x,y-10,strcat('Region ',num2str(i)),'color',cc_boxes(i,:));
    
    x=round(x); % Rounding off the x coordinate to get an integer value that will correspond to a pixel
    y=round(y); % Rounding off the y coordinate to get an integer value that will correspond to a pixel
    
    x_boxes(i) = x; % Storing the x coordinate of the top left corner of the box
    y_boxes(i) = y; % Storing the y coordinate of the top left corner of the box
    
    %%%%%%%
    % 
    
    
    
    %%%%%%%
    % Showing overlayed boxes in figure 2 and storing raw and normalized
    % voxel values for each channel in each ROI (box)
    for m = 1:Mchannels
        
%         figure(2) % Showing overlayed boxes in all channels of figure 2
%         subplot(1,Mchannels+1,m),
%         if (i==1)
%             rectangle('Position',[x,y,w,h], 'LineWidth', 1, 'EdgeColor','w') % Showing the box with white color
%         else
%             rectangle('Position',[x,y,w,h], 'LineWidth', 1, 'EdgeColor',cc_boxes(i,:)) % Showing the box
%         end
        
        Chm = imread(A, Chindices(m), 'Info', info); %Image in channel m
        ChmSIGG= Chm(y:y+(h-1),x:x+(w-1)); % Extracting the box from channel 1
        ChmA = imresize(double(ChmSIGG),Voxelsize); % Resizing the Channel m box depending upon the voxel size chosen
        ChmSIG = reshape(ChmA,[numel(ChmA),1]); % Converting the 2D matrix of intensity values into a 1D vector
        
        ChSIG(:,m,i) = ChmSIG; % Store the raw channel values
        ChSIGn(:,m,i) = (ChmSIG - minvalueChs(m))/(maxvalueChs(m) - minvalueChs(m)); % Store the normalized channel values
        
        clear Chm, clear ChmSIGG, clear ChmA, clear ChmSIG % Clearing temporary variable for reading another channel on next iteration of m
    end
    
    
%     figure(2) % Showing overlayed boxes in all channels of figure 2
%     subplot(1,Mchannels+1,Mchannels+1),
%     if (i==1)
%         rectangle('Position',[x,y,w,h], 'LineWidth', 1, 'EdgeColor','w') % Showing the box with white color
%     else
%         rectangle('Position',[x,y,w,h], 'LineWidth', 1, 'EdgeColor',cc_boxes(i,:)) % Showing the box
%     end
    
    %%%%%%%
    
    
    
end



figure(3)
saveas(gcf, strcat(filenameprefix,'-03-regions.pdf')); %Example: Saved as: zfish-regions.pdf

display('')
display('Step: Select anatomical regions of interest .... Completed')
display(strcat('File :',filenameprefix,'-03-regions.pdf is saved'))


%******************************************************** 
%******** Plot pairwise expression scatter plots.

ChannelSubplotBookkeeping = zeros(NScatterPlots,2); % It stores what subplot corresponds to which m (x-axis) and k (y-axis) channels e.g. [m k]

for i=1:Nboxes;
    
    x= x_boxes(i) ; % Rounded off x coordinate of the top left corner of the box 
    y= y_boxes(i); % Rounded off y coordinate of the top left corner of the box 
 
    
    %%%%%%%%
    % Generating pairwise scatter Plots
    subplotindexfig34 = 0;
    for m = 1:Mchannels
        for k = m+1:Mchannels
            
            subplotindexfig34 = subplotindexfig34 + 1;
            ChannelSubplotBookkeeping(subplotindexfig34,:) = [m k];
            
            figure(4) % Display all pairwise raw scatter plots for all ROIs on the same plot but with different color
            % Raw voxel intensities for each pairwise combination of analysis channels. Voxel intensities are shaded by anatomical region of interest. Background and maximum values used for normalization are depicted with dashed lines (see screenshot of Fig- ure S2C).
            set(gcf,'Units','inches')
            pos = get(gcf,'pos');
            set(gcf,'pos',[pos(1) pos(2) figurewidthinches 5])
            
            subplot(1,NScatterPlots,subplotindexfig34);
            plot(ChSIG(:,m,i),ChSIG(:,k,i),dotsymbol,'color',cc_boxes(i,:)), hold on, grid on,
            axis square, axis equal,axis([RawScatterplotrange(1) RawScatterplotrange(2) RawScatterplotrange(1) RawScatterplotrange(2)])
            xlabel(strcat('Ch',num2str(m),':',Chlabels{m}),'FontSize',6), ylabel(strcat('Ch',num2str(k),':',Chlabels{k}),'FontSize',6)
            title(strcat('Scatter plot:',num2str(subplotindexfig34)))
            set(gca,'XTick',[0 1000 2000 3000 4000])
            set(gca,'XTickLabel',{'0','','','','4000'})
            set(gca,'FontSize',6);
            set(gca,'YTick',[0 1000 2000 3000 4000])
            set(gca,'YTickLabel',{'0','','','','4000'})
            set(gca,'FontSize',6);
            
          

            minmaxlines = RawScatterplotrange(1):RawScatterplotrange(2);
            plot(minvalueChs(m)*ones(size(minmaxlines)),minmaxlines,'--','color',[50 50 50]/255,'LineWidth',0.5), hold on, grid on % Line for min value on x axis
            plot(maxvalueChs(m)*ones(size(minmaxlines)),minmaxlines,'--','color',[50 50 50]/255,'LineWidth',0.5), hold on, grid on % Line for max value on x axis
            plot(minmaxlines,minvalueChs(k)*ones(size(minmaxlines)),'--','color',[50 50 50]/255,'LineWidth',0.5), hold on, grid on % Line for min value on y axis
            plot(minmaxlines,maxvalueChs(k)*ones(size(minmaxlines)),'--','color',[50 50 50]/255,'LineWidth',0.5), hold on, grid on % Line for max value on y axis
        

            
            figure(5) % Display all pairwise normalized scatter plots for all ROIs on the same plot but with different color
            % Normalized voxel intensities for each pair-wise combination of analysis channels. Voxel intensities are shaded by anatomical region of interest (see screenshot of Figure S2D).
            set(gcf,'Units','inches')
            pos = get(gcf,'pos');
            set(gcf,'pos',[pos(1) pos(2) figurewidthinches 5])
            
            subplot(1,NScatterPlots,subplotindexfig34);
            plot(ChSIGn(:,m,i),ChSIGn(:,k,i),dotsymbol,'color',cc_boxes(i,:)), hold on, grid on,
            axis square, axis equal,axis([0 1 0 1])
            xlabel(strcat('Ch',num2str(m),':',Chlabels{m}),'FontSize',6), ylabel(strcat('Ch',num2str(k),':',Chlabels{k}),'FontSize',6)
            title(strcat('Scatter plot:',num2str(subplotindexfig34)))
            set(gca,'XTick',[0 0.2 0.4 0.6 0.8 1])
            set(gca,'XTickLabel',{'0','','','','','1'})
            set(gca,'FontSize',6);
            set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1])
            set(gca,'YTickLabel',{'0','','','','','1'})
            set(gca,'FontSize',6);
            
        
        end
    end
end

figure(4)
fig4=figure(4);
fig4.Renderer='Painters';
saveas(gcf, strcat(filenameprefix,'-04-rawscatterplots.pdf')); %Example: Saved as: zfish-rawscatterplots.pdf

figure(5)
saveas(gcf, strcat(filenameprefix,'-05-readout.pdf')); %Example: Saved as: zfish-readout.pdf

display('')
display('Step: Plot pairwise expression scatter plots .... Completed')
display(strcat('File :',filenameprefix,'-04-rawscatterplots.pdf is saved'))
display(strcat('File :',filenameprefix,'-05-readout.pdf is saved'))

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ?  Read-in from expression space to anatomical space.


%******************************************************** 
%******** Prompt a message to the user to specify Enter the subplot number of the pairwise scatter plot that you want to read-in and the number of polygons to be
% drawn in that pairwise scatter plot
prompt={'Enter the subplot number of the pairwise scatter plot that you want to read-in', 'Enter the number of polygons you want to draw on this paiwise scatter plot'};
prompt_title = 'Select expression clusters of interest';
defaultans = cat(2,{num2str(randi([1 NScatterPlots],[1,1]))},{num2str(randi([1 4],[1,1]))});
options.Interpreter = 'tex';
answer3 = inputdlg(prompt,prompt_title,[1 40],defaultans,options);

SubplotChosen = str2num(answer3{1}); % subplot number of the pairwise scatter plot that you want to read-in
polygonCounts = str2num(answer3{2}); % Number of polygons you want in that pairwise scatter plot
cc  = jet(polygonCounts+4);
cc(1,:) = [0 255 255]/255; % cyan
%cc(2,:) = [255 215 0]/255; % yellow
cc(2,:) = [203 249 8]/255;
cc(3,:) = [218 179 255]/255; % purple
cc(4,:) = [255 153 0]/255; % orange
cc(polygonCounts+4,:) = [100 100 100]/255; % gray

%******************************************************** 
%******** Select expression clusters of interest
% Use the cursor to define a polygonal boundary for each expression cluster (see screenshot of Figure S3A) in any of the normalized expression scatter plots (Matlab Figure 4). Each voxel can be selected in at most one expression cluster. Each cluster boundary can be specified in any of the pairwise axes. Voxels are shaded by anatomical region (see screenshot of Figure S3B).

%%%%%%%%
% Generating magnified version of the selected pairwise scatter plot

%TotalPolygonsDrawn = 0;

for i=1:Nboxes;
    figure(6) % Displays zoomed view of any one pairwise normalized scatter plots for all ROIs on the same plot but with different color
    set(gcf,'Units','inches')
    pos = get(gcf,'pos');
    set(gcf,'pos',[pos(1) pos(2) 5 5])
    
    m = ChannelSubplotBookkeeping(SubplotChosen,1);
    k = ChannelSubplotBookkeeping(SubplotChosen,2);
    plot(ChSIGn(:,m,i),ChSIGn(:,k,i),dotsymbol,'MarkerSize',10,'color',cc_boxes(i,:)), hold on, grid on,
    axis square, axis equal,axis([-0.1 1.2 -0.1 1.2])
    xlabel(strcat('Ch',num2str(m),':',Chlabels{m}),'FontSize',10), ylabel(strcat('Ch',num2str(k),':',Chlabels{k}),'FontSize',10)
    title(strcat('You have chosen Scatter plot:',num2str(SubplotChosen),'to draw ', num2str(polygonCounts),' expression clusters'))
    set(gca,'XTick',[0 0.2 0.4 0.6 0.8 1])
    set(gca,'XTickLabel',{'0','','','','','1'})
    set(gca,'FontSize',6);
    set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1])
    set(gca,'YTickLabel',{'0','','','','','1'})
    set(gca,'FontSize',6);
end


%%%%%%%%
% Drawing polygons on the selected pairwise scatter plot
   
PolygonsVertices_x = zeros(polygonCounts,100);
PolygonsVertices_y = zeros(polygonCounts,100);
PolygonsVertices_Count = zeros(polygonCounts,1);


for p = 1:polygonCounts
    figure(6)
    [x_poly_1,y_poly_1] = ginput;
    x_poly = [x_poly_1' x_poly_1(1)]; % to close the polygon
    y_poly = [y_poly_1' y_poly_1(1)]; % to close the polygon
    
    PolygonsVertices_Count(p) = length(x_poly);
    PolygonsVertices_x(p,1:length(x_poly)) = x_poly;
    PolygonsVertices_y(p,1:length(y_poly)) = y_poly;
    
    clear x_poly_1, clear y_poly_1
    
%     TotalPolygonsDrawn = TotalPolygonsDrawn +1;
    
    plot(x_poly,y_poly,'-','color',cc(p,:),'LineWidth',2), hold on,
     
    
    clear x_poly, clear y_poly
    
end



figure(6)
saveas(gcf, strcat(filenameprefix,'-06-cluster-selection.pdf')); %Example: Saved as: zfish-06-cluster-selection.pdf

display('')
display('Step: Select expression clusters of interest .... Completed')
display(strcat('File :',filenameprefix,'-06-cluster-selection.pdf is saved'))
display('')

%%%%%%%%
% Generating expression clusters of interest (voxels shaded by expression cluster)   
    
for i=1:Nboxes;

    subplotindexfig34 = 0;
    mx = ChannelSubplotBookkeeping(SubplotChosen,1);
    ky = ChannelSubplotBookkeeping(SubplotChosen,2);
            Ch_mx = ChSIGn(:,mx,i);
            Ch_ky = ChSIGn(:,ky,i);
    
    for m = 1:Mchannels-1
        
        for k = m+1:Mchannels
            
            subplotindexfig34 = subplotindexfig34 + 1;
            Ch_x = ChSIGn(:,m,i);
            Ch_y = ChSIGn(:,k,i);
            
            SelectedIndices = [];

            for p = 1:polygonCounts
                
                x_poly = PolygonsVertices_x(p,1:PolygonsVertices_Count(p)) ;
                y_poly = PolygonsVertices_y(p,1:PolygonsVertices_Count(p)) ;
                
                %in_poly = inpolygon(Ch_x,Ch_y,x_poly,y_poly);
                in_poly = inpolygon(Ch_mx,Ch_ky,x_poly,y_poly);

                SelectedIndices = [SelectedIndices find(in_poly==1)'];
                
                figure(7) % expression clusters of interest (voxels shaded by expression cluster). The same expression clusters are displayed with all voxels shaded by expression cluster. Voxels that are not selected are displayed with a separate color.
                set(gcf,'Units','inches')
                pos = get(gcf,'pos');
                set(gcf,'pos',[pos(1) pos(2) figurewidthinches 5])
                
                subplot(1,NScatterPlots,subplotindexfig34);
                plot(Ch_x(in_poly),Ch_y(in_poly),dotsymbol,'color',cc(p,:)), hold on, grid on,
                axis square, axis equal,axis([0 1 0 1])
                xlabel(strcat('Ch',num2str(m),':',Chlabels{m}),'FontSize',6), ylabel(strcat('Ch',num2str(k),':',Chlabels{k}),'FontSize',6)
                title(strcat('Scatter plot:',num2str(subplotindexfig34)))
                set(gca,'XTick',[0 0.2 0.4 0.6 0.8 1])
                set(gca,'XTickLabel',{'0','','','','','1'})
                set(gca,'FontSize',6);
                set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1])
                set(gca,'YTickLabel',{'0','','','','','1'})
                set(gca,'FontSize',6);
                
                %%%%%%%%
                % Generating masks for images based on polygons. This will
                % take care of the channel #s 1 through Mchannels-1
                Ch_Mask = Ch_x;
                Ch_Mask(in_poly)=1;
                Ch_Mask(~in_poly)=0;
%                 Ch_Mask = zeros(size(Ch_x));
%                 Ch_Mask(find(in_poly==1))=1;
%                 Ch_Mask(find(in_poly==1))=0;
                Ch_Masked = reshape(Ch_x.*Ch_Mask,[ceil(h*Voxelsize), ceil(w*Voxelsize)]);
                
                figure(80+i)
                set(gcf,'Units','inches')
                pos = get(gcf,'pos');
                set(gcf,'pos',[pos(1) pos(2) figurewidthinches figureheightinches])
                
                %                 subplot(Mchannels,polygonCounts+1,(m-1)*(polygonCounts+1) + p),
                %                 imshow(flipdim(Ch_Masked,1),[0 1]), hold on
                subplot(Mchannels,polygonCounts+2,(m-1)*(polygonCounts+2) + 1),
                %imshow(flipdim(reshape(Ch_x,[ceil(h*Voxelsize), ceil(w*Voxelsize)]),1),[0 1]), hold on
                imshow(reshape(Ch_x,[ceil(h*Voxelsize), ceil(w*Voxelsize)]),[0 1]), hold on
                ylabel(strcat('Ch',num2str(m),':',Chlabels{m}),'FontSize',8)
                
                subplot(Mchannels,polygonCounts+2,(m-1)*(polygonCounts+2) + p+1),
                %imshow(flipdim(Ch_Masked,1),[0 1]), hold on
                imshow(Ch_Masked,[0 1]), hold on
                
                %annotation(figure(80+i),'rectangle',[p/(polygonCounts+3)+(0.1)*(p-1) 0.1 1/(polygonCounts+3) 0.8],'color',cc(p,:),'Linewidth',2);
                
                if (m==1)
                    subplot(Mchannels,polygonCounts+2,(m-1)*(polygonCounts+2) + p+1),title(strcat('Cluster',num2str(p)),'color',cc(p,:))
                end
                
                if (m == (Mchannels-1))&&(k == Mchannels)
                    %%%%%%%%
                    % Generating masks for images based on polygons. This
                    % will take care of the channel # Mchannels 
                    Ch_Mask = Ch_y;
                    Ch_Mask(in_poly)=1;
                    Ch_Mask(~in_poly)=0;
%                     Ch_Mask = zeros(size(Ch_y));
%                     Ch_Mask(find(in_poly==1))=1;
%                     Ch_Mask(find(in_poly==1))=0;
                    Ch_Masked = reshape(Ch_y.*Ch_Mask,[ceil(h*Voxelsize), ceil(w*Voxelsize)]);
                    
                    figure(80+i)
                    set(gcf,'Units','inches')
                    pos = get(gcf,'pos');
                    set(gcf,'pos',[pos(1) pos(2) figurewidthinches figureheightinches])
                    
                    %                     subplot(Mchannels,polygonCounts+1,(Mchannels-1)*(polygonCounts+1) + p),
                    %                     imshow(flipdim(Ch_Masked,1),[0 1]), hold on
                    subplot(Mchannels,polygonCounts+2,(Mchannels-1)*(polygonCounts+2) + 1),
                    %imshow(flipdim(reshape(Ch_y,[ceil(h*Voxelsize), ceil(w*Voxelsize)]),1),[0 1]), hold on
                    imshow(reshape(Ch_y,[ceil(h*Voxelsize), ceil(w*Voxelsize)]),[0 1]), hold on
                    ylabel(strcat('Ch',num2str(k),':',Chlabels{k}),'FontSize',8)
                    
                    subplot(Mchannels,polygonCounts+2,(Mchannels-1)*(polygonCounts+2) + p+1),
                    %imshow(flipdim(Ch_Masked,1),[0 1]), hold on
                    imshow(Ch_Masked,[0 1]), hold on
                end
                
                
                
                
            end
            
            Ch_dummy = zeros(size(Ch_x));
            Ch_dummy(SelectedIndices)=1;
            UnselectedIndices = find(Ch_dummy == 0);
            
            clear Ch_dummy, clear SelectedIndices
            
            figure(7) % Plotting the voxels that are not selected are displayed with a separate color.
            
            subplot(1,NScatterPlots,subplotindexfig34);
            plot(Ch_x(UnselectedIndices),Ch_y(UnselectedIndices),dotsymbol,'color',cc(polygonCounts+4,:)), hold on, grid on,
            
            %%%%%%%%
            % Generating masks for images based on unselected voxels. This will
                % take care of the channel #s 1 through Mchannels-1
            %Ch_Mask = Ch_x;
            Ch_Mask = zeros(size(Ch_x));
            Ch_Mask(UnselectedIndices)=1;
            Ch_Mask(~UnselectedIndices)=0;
            Ch_Masked = reshape(Ch_x.*Ch_Mask,[ceil(h*Voxelsize), ceil(w*Voxelsize)]);
            
            figure(80+i)
%             subplot(Mchannels,polygonCounts+1,m*(polygonCounts+1)),
%             imshow(flipdim(Ch_Masked,1),[]), hold on
                
                subplot(Mchannels,polygonCounts+2,m*(polygonCounts+2)),
                %imshow(flipdim(Ch_Masked,1),[0 1]), hold on
                imshow(Ch_Masked,[0 1]), hold on
                if (m==1)
                    subplot(Mchannels,polygonCounts+2,m*(polygonCounts+2)),title('Not selected')
                    subplot(Mchannels,polygonCounts+2,1),title('All')
                end
            
            if (m == (Mchannels-1))&&(k == Mchannels)
                %%%%%%%%
                % Generating masks for images based on unselected voxels. This
                % will take care of the channel # Mchannels
                %Ch_Mask = Ch_y;
                Ch_Mask = zeros(size(Ch_y));
                Ch_Mask(UnselectedIndices)=1;
                Ch_Mask(~UnselectedIndices)=0;
                Ch_Masked = reshape(Ch_y.*Ch_Mask,[ceil(h*Voxelsize), ceil(w*Voxelsize)]);
                
                figure(80+i)
                set(gcf,'Units','inches')
                pos = get(gcf,'pos');
                set(gcf,'pos',[pos(1) pos(2) figurewidthinches figureheightinches])
                
%                 subplot(Mchannels,polygonCounts+1,Mchannels*(polygonCounts+1)),
%                 imshow(flipdim(Ch_Masked,1),[]), hold on
                subplot(Mchannels,polygonCounts+2,Mchannels*(polygonCounts+2)),
                %imshow(flipdim(Ch_Masked,1),[0 1]), hold on
                imshow(Ch_Masked,[0 1]), hold on
            end
            
            
            
            
        end
    end
    




















    
    figure(80+i)
    saveas(gcf, strcat(filenameprefix,'-08-readin-region',num2str(i),'.pdf')); %Example: Saved as: zfish-readin-region.pdf
      
    display(strcat('File :',filenameprefix,'-08-readin-region',num2str(i),'.pdf is saved'))   

    clear SelectedIndices

end
       
figure(7)
saveas(gcf, strcat(filenameprefix,'-07-clusters.pdf')); %Example: Saved as: zfish-07-clusters.pdf
    
display('')
display(strcat('File :',filenameprefix,'-07-clusters.pdf is saved'))                
                
display('')
display('Step: Display segmented images .... Completed')
  











%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ? Subcircuit expression scatter plots.

%******************************************************** 
%******** Prompt a message to the user to specify number of subcircuits
prompt={'Enter the number of subcircuits you want to define'};
prompt_title = 'Subcircuit Count';
defaultans = {num2str(randi([0 3],[1,1]))};
options.Interpreter = 'tex';
answer4 = inputdlg(prompt,prompt_title,[1 40],defaultans,options);

N_Subcircuit = str2num(answer4{1}); % Number of polygons you chose in each quadrant


%******************************************************** 
%******** Prompt a message to the user to define the subcircuits. Assign each analysis channel to at most one subcircuit; if two or more subcircuits are defined, the script will plot expression scatter plots for all pairwise combinations of subcircuit axes; if only one subcircuit is defined, no subcircuit scatter plots will be generated.
if (N_Subcircuit>1)
    prompt = {};
    defaultans = {};
    for sc = 1: N_Subcircuit
        prompt_text = strcat('Define the subcircuit #',num2str(sc));
        prompt = cat(2,prompt,{prompt_text});
        
        defaultans_text = strcat('[',num2str(randi([1 Mchannels],[1,randi([1 3],[1,1])])),']');
        defaultans = cat(2,defaultans,{defaultans_text});
    end
    
    prompt_title = 'Subcircuit definition';
    options.Interpreter = 'tex';
    CircuitGroups = inputdlg(prompt,prompt_title,[1 40],defaultans,options);  
end

display('')
display('Step: Define subcircuits .... Completed')


%******************************************************** 
%******** Calculate Subcircuits

if (N_Subcircuit>1)
    CircuitGroup_SIG = zeros(ceil(w*Voxelsize)*ceil(h*Voxelsize),N_Subcircuit,Nboxes); % This stores the normalized cicuit values for all the defined circuits and selected ROIs (boxes)
    CircuitGroup_labels = {};
    
    for i=1:Nboxes;
        
        for sc = 1: N_Subcircuit
            CircuitGroup1 = str2num(CircuitGroups{sc});
            dist_CircuitGroup1 = zeros(size(ChSIGn(:,1,1)));
            CircuitGroup1_labelcaption = '';
            
            for j = 1:numel(CircuitGroup1)
                dist_CircuitGroup1 = dist_CircuitGroup1 + ChSIGn(:,CircuitGroup1(j),i).^2;
                CircuitGroup1_labelcaption = strcat(CircuitGroup1_labelcaption,'/',Chlabels(CircuitGroup1(j)));
            end
            
            dist_CircuitGroup1 = sqrt(dist_CircuitGroup1/numel(CircuitGroup1));
            CircuitGroup_SIG(:,sc,i) = dist_CircuitGroup1;
            CircuitGroup1_labelcaption = char(CircuitGroup1_labelcaption);
            %CircuitGroup_labels{sc} = CircuitGroup1_labelcaption;
            CircuitGroup_labels{sc} = strcat(CircuitGroup1_labelcaption(2:length(CircuitGroup1_labelcaption)),' subcircuit intensity');
            clear CircuitGroup1, clear dist_CircuitGroup1, clear CircuitGroup1_labelcaption
        end
    end
end


%******************************************************** 
%******** Generate Subcircuit plots

if (N_Subcircuit>1)
    
    NCircuitPlots = nchoosek(N_Subcircuit,2);
    
        for i=1:Nboxes;
            mx = ChannelSubplotBookkeeping(SubplotChosen,1);
            ky = ChannelSubplotBookkeeping(SubplotChosen,2);
            Ch_mx = ChSIGn(:,mx,i);
            Ch_ky = ChSIGn(:,ky,i);
            
            subplotindexfig910 = 0;
            
            for sc1 = 1: N_Subcircuit-1
                for sc2 = sc1+1: N_Subcircuit
                    
                 subplotindexfig910 = subplotindexfig910 + 1;   
                 Ch_x = CircuitGroup_SIG(:,sc1,i);
                 Ch_y = CircuitGroup_SIG(:,sc2,i);
                 
            
                 figure(9) % expression clusters of interest (voxels shaded by expression cluster). The same expression clusters are displayed with all voxels shaded by expression cluster. Voxels that are not selected are displayed with a separate color.
                 set(gcf,'Units','inches')
                 pos = get(gcf,'pos');
                 set(gcf,'pos',[pos(1) pos(2) figurewidthinches 5])
                 
                 subplot(1,NCircuitPlots,subplotindexfig910);
                 plot(Ch_x,Ch_y,dotsymbol,'MarkerSize',10,'color',cc_boxes(i,:)), hold on, grid on,
                 axis square, axis equal,axis([0 1 0 1])
                 xlabel(CircuitGroup_labels{sc1},'FontSize',10), ylabel(CircuitGroup_labels{sc2},'FontSize',10)
                 set(gca,'XTick',[0 0.2 0.4 0.6 0.8 1])
                 set(gca,'XTickLabel',{'0','','','','','1'})
                 set(gca,'FontSize',6);
                 set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1])
                 set(gca,'YTickLabel',{'0','','','','','1'})
                 set(gca,'FontSize',6);
                 
                 
                 
                 SelectedIndices = [];
                 
                 for p = 1:polygonCounts
                     
                     x_poly = PolygonsVertices_x(p,1:PolygonsVertices_Count(p)) ;
                     y_poly = PolygonsVertices_y(p,1:PolygonsVertices_Count(p)) ;
                     
                     %in_poly = inpolygon(Ch_x,Ch_y,x_poly,y_poly);
                      in_poly = inpolygon(Ch_mx,Ch_ky,x_poly,y_poly);
                     
                     SelectedIndices = [SelectedIndices find(in_poly==1)'];
                     
                     figure(10) % expression clusters of interest (voxels shaded by expression cluster). The same expression clusters are displayed with all voxels shaded by expression cluster. Voxels that are not selected are displayed with a separate color.
                     set(gcf,'Units','inches')
                     pos = get(gcf,'pos');
                     set(gcf,'pos',[pos(1) pos(2) figurewidthinches 5])
                     
                     subplot(1,NCircuitPlots,subplotindexfig910);
                     plot(Ch_x(in_poly),Ch_y(in_poly),dotsymbol,'MarkerSize',10,'color',cc(p,:)), hold on, grid on,
                     axis square, axis equal,axis([0 1 0 1])
                     xlabel(CircuitGroup_labels{sc1},'FontSize',10), ylabel(CircuitGroup_labels{sc2},'FontSize',10)
                     set(gca,'XTick',[0 0.2 0.4 0.6 0.8 1])
                     set(gca,'XTickLabel',{'0','','','','','1'})
                     set(gca,'FontSize',6);
                     set(gca,'YTick',[0 0.2 0.4 0.6 0.8 1])
                     set(gca,'YTickLabel',{'0','','','','','1'})
                     set(gca,'FontSize',6);
                     
                 end
                 
                 Ch_dummy = zeros(size(Ch_x));
                 Ch_dummy(SelectedIndices)=1;
                 UnselectedIndices = find(Ch_dummy == 0);
                 
                 clear Ch_dummy, clear SelectedIndices
                 
                 figure(10) % Plotting the voxels that are not selected are displayed with a separate color.
                 
                 subplot(1,NCircuitPlots,subplotindexfig910);
                 plot(Ch_x(UnselectedIndices),Ch_y(UnselectedIndices),dotsymbol,'MarkerSize',10,'color',cc(polygonCounts+4,:)), hold on, grid on,
                 
                 
                    
                end
            end

        end   
end



figure(9)
saveas(gcf, strcat(filenameprefix,'-09-subcircuits-byregion.pdf')); %Example: Saved as: zfish-subcircuits-byregion.pdf

figure(10)
saveas(gcf, strcat(filenameprefix,'-10-subcircuits-bycluster.pdf')); %Example: Saved as: zfish-subcircuits-bycluster.pdf
    
display('')
display('Step: Subcircuit plots (shaded by anatomical region).... Completed')
display(strcat('File :',filenameprefix,'-09-subcircuits-byregion.pdf is saved'))        

display('')
display('Step: Subcircuit plots (shaded by expression cluster).... Completed')
display(strcat('File :',filenameprefix,'-10-subcircuits-bycluster.pdf is saved')) 

display('')
display('Script readoutreadin.m is completed')


 %%%%%%%%
%         % Generating CircuitDiagrams
%         qHCR_CircuitPlot(ChSIG,i,CircuitGroup1,CircuitGroup2,cc,6,Chindices,Chlabels,dotsymbol) %qHCR_CircuitPlot(ChSIG,i,CircuitGroup1,CircuitGroup2,cc,figureindex,Chindices,Chlabels,dotsymbol)
%         
%         
% 
% 
% 
% 
% % subcircuit definitions: assign each analysis channel to at most one subcircuit; if two or more subcircuits are defined, the script will plot expression scatter plots for all pairwise combinations of subcircuit axes; if only one subcircuit is defined, no subcircuit scatter plots will be generated.
% CircuitGroup1 = [1,2]; % Channels (from your original image) that are clubbed in the circuit on x axis
% CircuitGroup2 = [3,4]; % Channels (from your original image) that are clubbed in the circuit on y axis
% 
% 






